######################################################################################
######################################################################################

# E. RIVOT - June 2014

# Example of an R-code to fit a linear model with the library BRugs

# Compare a linear regression and a linear regression with residuals ~ AR1

######################################################################################
######################################################################################


library("BRugs") 	
library(MASS)
library(boot)

# load a function needed to save and plot mcmc results
source("f.save.mcmc.R")
source("f.density.bivar.R")

dir.create(path = c("res"), recursive = TRUE, showWarnings = TRUE)

# Load/check model, load data and inits

model <- "model_18years.txt"
model <- "model_18years_hier.txt"
model <- "model_18years_hier_Q.txt"

data <- "data.txt"

n <- source("data.txt")$value$n
n.pred <- source("data.txt")$value$n.pred
Q <- source("data.txt")$value$Q
Q.pred <- source("data.txt")$value$Q.pred


# inits1 <- "inits.txt" ; inits2 <- "inits.txt" ; inits3 <- "inits.txt"
inits1 <- "inits.logit.txt" ; inits2 <- "inits.logit.txt" ; inits3 <- "inits.logit.txt"

modelCheck(model) 				# Check the model
modelData(data) 					# Load the data
modelCompile(numChains=3) 			# Compile the model with 3 chains
modelInits(c(inits1, inits2, inits3)) 	# Load the three initialisations 
modelGenInits()

# Number of MCMC iterations

Nb.burn.in <- 10000 	# Number of iterations for Burn in
N.iter <- 10000 		# Number of iterations to be considered for inference

# Burn-in period

modelUpdate(Nb.burn.in)

# Set the quantities we are interested in

monitor <- c("theta", "N", "theta.pred", "theta.p", "a", "b")

samplesSet(monitor)
dicSet()
modelUpdate(N.iter) 			


# Have a look on posterior statistics

samplesStats("*") 

# Have a look on MCMC chains

windows()
Results <- samplesHistory("*", mfrow = c(4, 4))



# Store mcmc results

mcmc.theta <- as.data.frame(as.matrix(buildMCMC("theta")))
mcmc.N <- as.data.frame(as.matrix(buildMCMC("N")))

# Only for hierarchical models

mcmc.theta.hier <- as.data.frame(as.matrix(buildMCMC("theta")))
mcmc.N.hier <- as.data.frame(as.matrix(buildMCMC("N")))
a <- as.matrix(buildMCMC("a"))
b <- as.matrix(buildMCMC("b"))
theta.p <- as.matrix(buildMCMC("theta.p"))
mcmc.theta.pred <- as.data.frame(as.matrix(buildMCMC("theta.pred")))


# setwd("./res")
# for(i in 1:length(monitor)) {f.save.mcmc(var=monitor[i],write=TRUE)}
# a <- as.vector(read.table(file = "a", header = T, sep = ""))$a
# b <- as.vector(read.table(file = "b", header = T, sep = ""))$b
# setwd("../")


# ----------------------------------------------
# Joint posterior distribution for (theta,N)
# ----------------------------------------------

x <- mcmc.theta[,1]
y <- mcmc.N[,1]

def.par <- par(no.readonly = TRUE)
windows()

f.density.bivar(x,y,nlevels=3,nb.points=2000)
par(def.par)



# ----------------------------------------------
# Trap efficiency  = f(Q)
# ----------------------------------------------

windows()

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

size.labels = 1
size.text = 1.3
box.size1 = 1
box.size2 = 0.5
col1 <- "grey90"
col2 <- "grey55"
col1 = c(rep(col1, times=n))
col2 = c(rep(col2, times=n))

X1 <- mcmc.theta.hier

x <- Q
x.plot <- Q.pred

labels <- ""

x.min = 0 ; x.max = max(x.plot)+1
y.min = 0 ; y.max = 1
y.label = "trap. eff"
x.label = "water discharge"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = x[1], outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n)
{
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = x[i], add = T, outpch = NA, boxwex=box.size1, col = col1[i])
}
axis(side =1, at=signif(seq(x.min,x.max,length.out=11),digits=2), labels = T, las=1, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)



# ----------------------------------------------
# Trap efficiency // hier vs indep
# ----------------------------------------------

windows()

size.labels = 1
box.size = 0.5
size.text = 1.3
col1 <- "grey90"
col2 <- "grey55"
col1 = c(rep(col1, times=n))
col2 = c(rep(col2, times=n))

par(mfrow = c(1,1), bty="n")

X1 <- mcmc.theta
X2 <- mcmc.theta.hier

years <- 1986:(1986+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 1
y.label = "Trapping efficiency"
x.label = "Years"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size, col = col1[i]) }

for (i in 1:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size, col = col2[i]) }

axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)



# ----------------------------------------------
# Trap efficiency  +  Q.pred and prediction
# ----------------------------------------------

windows()

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

size.labels = 1
size.text = 1.3
box.size1 = 1
box.size2 = 0.5
col1 <- "grey90"
col2 <- "pink"
col1 = c(rep(col1, times=n))
col2 = c(rep(col2, times=n))

X1 <- mcmc.theta.hier
X2 <- mcmc.theta.pred

conf1l <- boxplot(X2)$stats[1,]
conf1u <- boxplot(X2)$stats[5,]
conf2l <- boxplot(X2)$stats[2,]
conf2u <- boxplot(X2)$stats[4,]

c <- median(a)
d <- median(b)
x <- Q
x.plot <- Q.pred
x.plot.inv <- sort(x.plot,decreasing=TRUE)
logit.theta.plot <- c*((x.plot-mean(Q))/sd(Q))+d
theta.plot <- exp(logit.theta.plot)/(1+exp(logit.theta.plot))

labels <- ""

x.min = 0 ; x.max = max(x.plot)+1
y.min = 0 ; y.max = 1
y.label = "trap. eff"
x.label = "water discharge"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = x[1], outpch = NA, boxwex=box.size1, col = col1[1]) 

polygon(c(x.plot,x.plot.inv),c(conf1l,conf1u[length(Q.pred):1]),col="grey80",border=NA)
polygon(c(x.plot,x.plot.inv),c(conf2l,conf2u[length(Q.pred):1]),col="grey60",border=NA)

for (i in 1:n)
{
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = x[i], add = T, outpch = NA, boxwex=box.size1, col = col1[i])
}

points(x.plot, theta.plot, type="l",lty=1, lwd=2, col="red")

axis(side =1, at=signif(seq(x.min,x.max,length.out=11),digits=2), labels = T, las=1, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)



