#####################################################################################
#####################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# Stock-recruitment analysis
# Application to the A. salmon SR data from the Margaree river (Nova Scotia, Canada)
# Ricker model LogNormal with "Natural" parameters
# Chapter 7 - Section 7.3.1

######################################################################################
######################################################################################

rm(list=ls())
library(boot)
library("BRugs")
library(MASS)




# ------------------------------------------------------------------------------------
# Draw SR the data
# ------------------------------------------------------------------------------------


data <- source("data_SRMargaree.txt")$value
S <- data$s
R <- data$r
n <- data$n

# windows()

res <- 6
name_figure <- "Margaree_SR_data.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), oma = c(1,1,1,1))
cize.text <- 1.3

plot(S,R, type="p", pch=21, bg="black",
	xlim=c(0,6000),ylim=c(0,8000), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")

points(0:6000,0:6000, type="l", lty = 2, lwd = 1)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Stock", bty="n", line=3, cex=cize.text, font = 1)
mtext(side=2, text = "Recruitment", bty="n", line=3, cex=cize.text, font = 1)

# legend(	legend = c("Prior","Posterior"),
# 		col = c("black","black"), 	
# 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 

dev.off()



# ------------------------------------------------------------------------------------
# MCMC config.
# ------------------------------------------------------------------------------------

# Number iteration for the "burn in" phase
n.burnIn <- 10000
# Number iteration for inferences
n.iter <- 10000
thin <- 10
# Number of chains
n.Chains <- 3


# ------------------------------------------------------------------------------------
# Run model
# ------------------------------------------------------------------------------------

# Load/check model, load data and inits

model <- "model_SRMargaree_Ricker_LogN_Natural.txt"
data <-  "data_SRMargaree.txt"

modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelGenInits()

# Burn in period

modelUpdate(numUpdates = n.burnIn, thin = 1) 

# Inferences

samplesSet(c("tau","alpha","beta","sigma","Smax","Rmax")) 
modelUpdate(numUpdates = n.iter, thin = thin) 


# -------------------------------------------------------------------------
# Have a quick look on results
# -------------------------------------------------------------------------

# Have a look on MCMC chains of parameters theta

windows()
Results <- samplesHistory("*", mfrow = c(3, 1))

# Get posterior statistics of parameters theta

samplesStats("*")                                                                  

